#include <vector>
#include <iostream>
#include <string>
#include <fstream>
#include <GL/glut.h>
#include "Vertex.h"
#include "RayTracer.h"
#include "Loader.h"
#include "ppmwriter.h"

RayTracer::Image gImage;
int gWidth = 1024;
int gHeight = 768;
std::string gScene;
int gOversample = 1;

void display() {
	glClear(GL_COLOR_BUFFER_BIT);
	glDrawPixels(gImage.width, gImage.height, GL_RGB, GL_UNSIGNED_BYTE, &*gImage.data.begin());
}

void keyboard(unsigned char key, int x, int y) {
	if (key == 'q' || key == 'Q') {
		exit(0);
	}
}

void mouse(int button, int down, int x, int y) {
	if (button == 0 && down) {
		Loader::process(gScene.c_str(), gWidth, gHeight, 1, x, y);
	}
}

void usage(const char *name) {
	std::cerr << "Usage: " << name << " scenefile width height -window [-aa n]\n";
	std::cerr << "       " << name << " scenefile width height -ppm imagefile.ppm [-aa n]\n";
}

int main(int argc, char *argv[]) {
	glutInit(&argc, argv);

	std::cout << "Cory's Ray Tracing Engine\n";

	if (argc < 4) {
		usage(argv[0]);
	} else {
		bool showWindow = false;
		std::string fileName = "";
		
		gScene = argv[1];

		gWidth = atoi(argv[2]);
		gHeight = atoi(argv[3]);

		for (int i = 4; i < argc; i++) {
			if (!strcmp(argv[i], "-window")) {
				showWindow = true;
			} else if (!strcmp(argv[i], "-ppm") && i < argc - 1) {
				fileName = argv[++i];
			} else if (!strcmp(argv[i], "-aa") && i < argc - 1) {
				gOversample = atoi(argv[++i]);
			} else {
				std::cerr << "Erroneous command-line argument: " << argv[i] << "\n";
			}
		}

		std::cout << "Generating...\n";
		gImage = Loader::process(gScene.c_str(), gWidth, gHeight, gOversample);
		std::cout << "Complete.\n";

		if (fileName.size()) {
			std::ofstream file(fileName.c_str());
			if (ppm_writer(file, &*gImage.data.begin(), gImage.width, gImage.height)) {
				std::cout << "File " << fileName << " written successfully.\n";
			} else {
				std::cout << "Error writing file " << fileName << ".\n";
			}
		}

		if (showWindow) {
			glutInitDisplayMode(GLUT_SINGLE|GLUT_RGB);
			glutInitWindowSize(gImage.width, gImage.height);
			glutCreateWindow(argv[0]);
			glutDisplayFunc(display);
			glutKeyboardFunc(keyboard);
			glutMouseFunc(mouse);
			glutMainLoop();
		}
	}

	std::cout << "Done.\n";

	return 0;
}
